# Eiko Fried, Network Analysis Workshop Odense, September 25/26 2017
# Slides, syntax and data for the workshop can be found at: www.eiko-fried.com/Odense2017
# More information: www.eiko-fried.com, Twitter @EikoFried, eikofried@gmail.com



############### MRF 1: network estimation and inference

# ---------------------------------------------------------------------------------------
# ---------- 1. Load libaries -----------------------------------------------------------
# ---------------------------------------------------------------------------------------
library("qgraph")
library("bootnet") 
library("dplyr")
library("NetworkComparisonTest")
library("mgm")

# ---------------------------------------------------------------------------------------
# ---------- 2. Adjacency Matrix --------------------------------------------------------
# ---------------------------------------------------------------------------------------

AdjacencyMatrix <- matrix(0,4,4)  # create matrix
AdjacencyMatrix[1,2] <- AdjacencyMatrix[2,1] <- AdjacencyMatrix[2,3] <- AdjacencyMatrix[3,2] <- AdjacencyMatrix[2,4] <- AdjacencyMatrix[4,2] <- 1  # fill it with values
AdjacencyMatrix 
qgraph(AdjacencyMatrix) # plot network from values of the matrix

# ---------------------------------------------------------------------------------------
# ---------- 3. Data preparation --------------------------------------------------------
# ---------------------------------------------------------------------------------------

# set working directly to where the data file is
load("data.Rdata")
View(data)

# the following 3 lines are to make the plots a bit fancier (to add a legend etc), you don't need this. 
names<-c("B1","B2", "B3", "B4", "B5", "C1", "C2", "D1", "D2", "D3", "D4", "D5", "D6", "D7", "E1", "E2", "E3", "E4", "E5", "E6")
longnames <- c("Intrusive thoughts", "Nightmares", "Flashbacks", "Emotional cue reactivity", "Physiological cue reactivity", "Avoidance of thoughts", "Avoidance of reminders", "Trauma-related amnesia", "Negative beliefs", "Blame of self or others", "Negative trauma-related emotions", "Loss of interest", "Detachment", "Restricted affect", "Irritability/anger", "Self-destructive/reckless behavior", "Hypervigilance", "Exaggerated startle response", "Difficulty concentrating", "Sleep disturbance")
gr <- list("Intrusions"=c(1:5), "Avoidance"=c(6:7), "Cognition & mood alterations"=c(8:14),"Arousal & reactivity alterations"=c(15:20)) 

# ---------------------------------------------------------------------------------------
# ---------- 4. Correlation network -----------------------------------------------------
# ---------------------------------------------------------------------------------------

corMat <- cor_auto(data)

# estimate appropriate correlations among items; in our case, these are polychoric
# sometimes there are problems with polychoric correlations, in which case you want to use Spearman via cor(data, method="spearman")
View(corMat)

g1 <- qgraph(corMat, layout="circle") # visualize correlation matrix

# add some graphical arguments defined in the beginning of the script, and plot it as PDF

pdf("correlation_network.pdf", width=7, height=5)
g1b <-qgraph(corMat, labels=names, layout="circle", vsize=6, border.width=1.5, border.color="black", 
             groups=gr, color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'), nodeNames = longnames,legend.cex=.35)
dev.off()

# ---------------------------------------------------------------------------------------
# ---------- 5. Fruchtermann-Reingold algorithm -----------------------------------------
# ---------------------------------------------------------------------------------------

corMat2 <- corMat
g1c <-qgraph(corMat2, labels=names, layout="spring", vsize=6, border.width=1.5, border.color="black", 
             groups=gr, color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'), nodeNames = longnames,legend.cex=.35)

corMat2[1,2] <- corMat2[2,1] <- corMat[1,2] + 0.0001 # we change the correlation matrix just a tiny little bit

# the visual result is very different now

layout(t(1:2))
g1d <-qgraph(corMat, layout="spring", vsize=6, border.width=1.5, border.color="black", 
             color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'))
g1e <-qgraph(corMat2,layout="spring", vsize=6, border.width=1.5, border.color="black", 
             color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'))

# the algorithm is chaotic, and the tiniest changes to the input matrix will lead to fairly different layouts
# we therefore constrain the layout to be equal

L1 <- averageLayout(corMat, corMat2)

layout(t(1:2))
g1d <-qgraph(corMat, layout=L1, vsize=6, border.width=1.5, border.color="black", 
             color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'))
g1e <-qgraph(corMat2,layout=L1, vsize=6, border.width=1.5, border.color="black", 
             color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'))

# ---------------------------------------------------------------------------------------
# ---------- 6. Visually compare networks -----------------------------------------------
# ---------------------------------------------------------------------------------------

### Obtain a correlation matrix with much smaller correlations
corMat3 <- corMat/10

layout(t(1:2))
g1f <-qgraph(corMat, layout=L1, cut=0)
g1g <-qgraph(corMat3, layout=L1, cut=0) # surprisingly, networks look the same ... because of scaling (R "zooms in" and scales everything accordingly)

layout(t(1:2))
g1h <-qgraph(corMat, layout=L1, details=T, cut=0) # the details=TRUE command helps us to see what the maximum edge is
g1i <-qgraph(corMat3, layout=L1, details=T, cut=0)

layout(t(1:2))
g1j <-qgraph(corMat, layout=L1, details=T, cut=0, maximum=.74) # and setting the maximum allows us visual comparison
g1jk <-qgraph(corMat3, layout=L1, details=T, cut=0, maximum=.74)

dev.off()

# ---------------------------------------------------------------------------------------
# ---------- 7. Partial correlation network ---------------------------------------------
# ---------------------------------------------------------------------------------------

pcor <- estimateNetwork(data, default="pcor")
pcor_plot <- plot(pcor, details=T, layout=L1, cut=0) # we obtain spurious negative relationships

# ---------------------------------------------------------------------------------------
# ---------- 8. Regularized partial correlation network ---------------------------------
# ---------------------------------------------------------------------------------------

GGM <- estimateNetwork(data, default="EBICglasso") # lasso regularized gaussian graphical model for ordinal or gaussian data

GGM_plot<-plot(GGM, labels=names, layout="spring", vsize=6, cut=0, border.width=1.5, border.color="black", 
         groups=gr, color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'), nodeNames = longnames,legend.cex=.4)

# let's look at the adjacency matrix

adjacencyMatrix <- GGM$graph
View(adjacencyMatrix)

# plot all 3 together
pdf("eiko.pdf", width=8, height=5)
layout(t(1:3))
qgraph(corMat, layout=L1, cut=0, details=T, title="Cor, Pcor, and GGM") # correlation network
qgraph(pcor_plot, layout=L1, details=T, cut=0) # partial correlation network
plot(GGM, layout=L1, details=T, cut=0) # GGM
dev.off()

# ---------------------------------------------------------------------------------------
# ---------- 9. Ising Model -------------------------------------------------------------
# ---------------------------------------------------------------------------------------

# we need to think about missing data, ising model cannot deal with missing data. options: listwise deletion? random forest imputation (e.g. packages MICE or missforest); multiple imputation (e.g. packages amelia)

data_bin <- apply(data, 2, function(x) ifelse(x > 1, 1, 0)) # we dichotomize data so we have an example case of binary data; we do this just to learn how to fit an Ising Model 

IsingModel <- estimateNetwork(data_bin, default="IsingFit") # Ising Model
IsingModel_plot<-plot(IsingModel, labels=names, layout=L1, vsize=6, cut=0, border.width=1.5, border.color="black", 
               groups=gr, color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'), nodeNames = longnames,legend.cex=.4)

layout(t(1:2))
plot(GGM, layout=L1, details=T, cut=0) # GGM
plot(IsingModel, layout=L1, details=T, cut=0) # Ising Model; edges not standardized.

# ---------------------------------------------------------------------------------------
# ---------- 10. Network inference -------------------------------------------------------
# ---------------------------------------------------------------------------------------

c1 <- centralityPlot(GGM) # estimate and plot centrality estimates for GGM network

# investigate three different metrics
centrality(GGM)$InDegree # unstandardized degree
scale(centrality(GGM)$InDegree) # standardized degree
centrality(GGM)$Closeness 
scale(centrality(GGM)$Closeness)
centrality(GGM)$Betweenness

# correlate centrality indices in case we're interested in that
cor(centrality(GGM)$InDegree, centrality(GGM)$Betweenness, 
    method = "spearman") 





############### MRF 2: network stability and replicability

# ---------------------------------------------------------------------------------------
# ---------- 1. Network stability -------------------------------------------------------
# ---------------------------------------------------------------------------------------

### get adjacency matrix
GGM$graph 

# assume we want to compare edges 13-14 and 14-9, see slides. 
GGM$graph[13,14] #.45
GGM$graph[14,9]  #.12

### bootstrapping
b1 <- bootnet(GGM, boots=1000, nCores=8)
b2 <- bootnet(GGM, boots=1000, nCores=8, type="case")
# alternatively, load bootstrapped data
load("b1.Rdata")
load("b2.Rdata")

### save output *immediately*
save(b1, file = "b1.Rdata")
save(b2, file = "b2.Rdata")

### Plot edge weight CI
pdf("Fig1.pdf")
plot(b1, labels = FALSE, order = "sample") 
dev.off()

### Edge weights diff test
pdf("Fig2.pdf")
plot(b1, "edge", plot = "difference", onlyNonZero = TRUE, order = "sample")
dev.off()

### Plot centrality stability
pdf("Fig3.pdf") 
plot(b2)
dev.off()

### Centrality stability coefficient
corStability(b2)  

### Centrality diff test
pdf("Fig4.pdf")
plot(b1, "strength", order="sample", labels=TRUE) 
dev.off()

# ---------------------------------------------------------------------------------------
# ---------- 2. Replicability -----------------------------------------------------------
# ---------------------------------------------------------------------------------------

### Obtain two different datasets:
data1 <- slice(data, c(1:110))     # n = 110
data2 <- slice(data, c(111:221))   # n = 111

### similarity: visual
network1 <- estimateNetwork(data1, default="EBICglasso")
network2 <- estimateNetwork(data2, default="EBICglasso")

graph1 <- plot(network1, layout="spring", cut=0)
graph2 <- plot(network2, layout="spring", cut=0)

L <- averageLayout(graph1,graph2)

layout(t(1:2))
graph1 <- plot(network1, layout=L, cut=0)
graph2 <- plot(network2, layout=L, cut=0)

dev.off() 

### similarity: statistical
cor(network1$graph[lower.tri(network1$graph)], 
    network2$graph[lower.tri(network1$graph)], 
    method="spearman")

cor(centrality(network1)$InDegree, centrality(network2)$InDegree)

### difference: statistical
nct_results <- NCT(data1, data2, it=1000, binary.data=FALSE, paired=FALSE, test.edges=TRUE, 
                   edges='all', progressbar=TRUE)
# alternatively, load 
load("NCT.Rdata")
save(nct_results, file="NCT.Rdata")

nct_results$nwinv.pval    #network invariance (structure)
nct_results$glstrinv.pval # global strength invariance (density)
nct_results$einv.pvals    # all individual edges
sum(nct_results$einv.pvals$"p-value" < 0.05) # show me all significant differences

?NCT  # <<<---- help is here!



############### MRF 3: mixed graphical models & predictability

# ---------------------------------------------------------------------------------------
# ---------- 1. Predictability ----------------------------------------------------------
# ---------------------------------------------------------------------------------------

fit <- mgm(data, type=rep('g',20), level=rep(1,20), k=2, 
           lambdaSel = 'EBIC', lambdaGam = 0)

fit_plot <- qgraph(fit$pairwise$wadj, cut=0, 
                    layout = L1, edge.color = fit$pairwise$edgecolor)

pred_obj <- predict(object = fit, 
                    data = data, 
                    errorCon = 'R2')
pred_obj$error

fit_plot2 <- qgraph(fit$pairwise$wadj, cut=0, pie = pred_obj$error[,2],
                    layout = L1, edge.color = fit$pairwise$edgecolor)

# ---------------------------------------------------------------------------------------
# ---------- 2. MGM ---------------------------------------------------------------------
# ---------------------------------------------------------------------------------------

# make first 2 variables categorical, next 3 binary, rest remains ordinal

data3 <- data
data3[[1]][data[[1]]==3] <- data3[[2]][data[[2]]==3] <- 2
data3[[1]][data[[1]]==4] <- data3[[2]][data[[2]]==4] <- 2
data4 <- apply(data3, 2, function(x) ifelse(x > 2, 1, 0))

data5 <- cbind(data3[,c(1:2)], data4[,c(3:5)], data3[,c(6:20)])

type <- c(rep('c', 5), rep('g', 15))  # c('c','c','c','c','c','g','g','g','g','g','g','g','g','g','g','g','g','g','g','g')
level <- c(rep(3, 2), rep(2, 3), rep(1, 15))

fit2 <- mgm(data5, type=type, level=level, k=2, lambdaSel = 'EBIC', 
           lambdaGam = 0.25, binarySign=TRUE)

fit_plot2 <- qgraph(fit2$pairwise$wadj, cut=0, 
      layout = L1, edge.color = fit2$pairwise$edgecolor)






# community detection add on
plot(eigen(cor_auto(data))$values, type="b")
abline(h=1,col="red", lty = 3)

library("igraph")
g = as.igraph(GGM, attributes=TRUE)
sgc <- spinglass.community(g)
sgc$membership

library("EGA")
ega<-EGA(data, plot.EGA = TRUE)
